//
// Tuple.cpp
//
// $Id: //poco/1.3/Data/samples/Tuple/src/Tuple.cpp#2 $
//
// This sample demonstrates the Data library.
//
// Copyright (c) 2007, Applied Informatics Software Engineering GmbH.
// All rights reserved.
//
// This is unpublished proprietary source code of Applied Informatics
// Software Engineering GmbH.
// The contents of this file may not be disclosed to third parties, 
// copied or duplicated in any form, in whole or in part, without
// prior written permission from Applied Informatics.
//


#include "Poco/SharedPtr.h"
#include "Poco/Tuple.h"
#include "Poco/Data/SessionFactory.h"
#include "Poco/Data/Session.h"
#include "Poco/Data/SQLite/Connector.h"
#include <vector>
#include <iostream>


using namespace Poco::Data;


int main(int argc, char** argv)
{
	typedef Poco::Tuple<std::string, std::string, int> Person;
	typedef std::vector<Person> People;

	// register SQLite connector
	Poco::Data::SQLite::Connector::registerConnector();
	
	// create a session
	Session session("SQLite", "sample.db");

	// drop sample table, if it exists
	session << "DROP TABLE IF EXISTS Person", now;
	
	// (re)create table
	session << "CREATE TABLE Person (Name VARCHAR(30), Address VARCHAR, Age INTEGER(3))", now;
	
	// insert some rows
	People people;
	people.push_back(Person("Bart Simpson",	"Springfield", 12));
	people.push_back(Person("Lisa Simpson",	"Springfield", 10));
	
	Statement insert(session);
	insert << "INSERT INTO Person VALUES(?, ?, ?)",
		use(people), now;
	
	people.clear();

	// a simple query
	Statement select(session);
	select << "SELECT Name, Address, Age FROM Person",
		into(people),
		now;
	
	for (People::const_iterator it = people.begin(); it != people.end(); ++it)
	{
		std::cout << "Name: " << it->get<0>() << 
			", Address: " << it->get<1>() << 
			", Age: " << it->get<2>() <<std::endl;
	}

	return 0;
}
